function [Fr,Pr,Er,rv,designinfo]=oflqr(sys,Q,R,N,ct,varargin) 
%OFLQR Output-feedback LQR design for LTI ss systems.
%    [F,P,E,rv,dinfo] = OFLQR(sys,Q,R,N,ct,Opt) calculates the (sub)optimal
%    robust structurerd output-feedback gain matrix F such that:
%
%       * For a continuous-time polytopic state-space model SYS, the 
%         output-feedback law defined with ct (P, PI, PID, PD, D or DOF, 
%         DOFI, DOFID, DOFD) guarantees the robust closed-loop stability 
%         (quadratic stability) and minimizes the cost function:
%
%               J = Integral {x'Qx + u'Ru + 2x'Nu} dt
%
%         Subject to the system dynamics  dx/dt = Aj x + Bj u;  y = C x;
%           yi = Ci x; yd = Cd x              j = 1,2,...,p
%
%           x - state vector          u - control input vector
%           y - output vector        
%          yi - output vector for integral part 
%          yd - output vector for derivative part 
%
%     INPUTS:
%       REQUIRED
%        SYS    - state-space LTI systems (in convex polytopic form)
%                   > array of ss models: sys(:,:,1:p), (Matlab)
%                   > or cels of ss objects: sys{1:p}   (Matlab, Octave)
%                   > or single ss object: sys          (Matlab, Octave)
%                 p - number of LTI systems
%        Q      - weighting matrix related to states (Q >= 0 if N == 0)
%        R      - positive definite weighting matrix (R > 0)
%        N      - If N~=0 then Q-N*inv(R)*N'>=0 (use eig to check)
%        ct     - controller type:      
%                 ct='p'   - Proportional (P) controller
%                           F = [Kp]
%                           u = - F y  
%                 ct='pi'  - Proportional-Integral (PI) controller
%                           F = [Kp,Ki]
%                           u = - Kp y - Ki Integral yi dt   
%                 ct='pid' - Proportional-Integral-Derivative (PID) controller    
%                           F = [Kp,Ki,Kd]
%                           u = - Kp y - Ki Integral yi dt - Kd dydf/dt
%                           where dydf/dt is the vector of filtered 
%                           derivatives, using derivative filter
%                           Gf(s)=Nf s/(s+Nf). (default Nf=100)
%                 ct='pd'  - Proportional-Derivative (PD) controller
%                           F = [Kp,Kd]
%                           u = - Kp y - Kd dydf/dt
%                           where dydf/dt is the vector of filtered 
%                           derivatives, using derivative filter
%                           Gf(s)=Nf s/(s+Nf). (default Nf=100)
%                 ct='d'   - Derivative (D) controller
%                           F = [Kd]
%                           u = - Kd dydf/dt
%                           where dydf/dt is the vector of filtered 
%                           derivatives, using derivative filter
%                           Gf(s)=Nf s/(s+Nf). (default Nf=100)
%                 ct='dof' - Dynamic output-feedback with order nc
%                            (default: nc=2)
%                         F = [Dc,Cc;Bc,Ac]
%                       dxc = - Ac xc - Bc y
%                         u = - Cc xc - Dc y
%                 ct='dofi' - Dynamic output-feedback with Integral part 
%                             with order nc (default: nc=2)
%                         F = [Dc1,Cc,Dc2;Bc1,Ac,Bc2]
%                       dxc = - Ac xc - Bc1 y - Bc2 Integral yi
%                         u = - Cc xc - Dc1 y - Dc2 Integral yi
%                 ct='dofid' - Dynamic output-feedback with Integral and 
%                              filtered derivative part
%                              with order nc (default: nc=2)
%                       F = [Dc1,Cc,Dc2,Dc3;Bc1,Ac,Bc2,Bc3]
%                     dxc = - Ac xc - Bc1 y - Bc2 Integral yi - Bc3 dydf/dt
%                       u = - Cc xc - Dc1 y - Dc2 Integral yi - Dc3 dydf/dt
%                              where dydf/dt is the vector of filtered 
%                              derivatives, using derivative filter
%                              Gf(s)=Nf s/(s+Nf). (default Nf=100)
%                 ct='dofd' - Dynamic output-feedback with filtered 
%                             derivative part
%                             with order nc (default: nc=2)
%                         F = [Dc1,Cc,Dc3;Bc1,Ac,Bc3]
%                       dxc = - Ac xc - Bc1 y - Bc3 dydf/dt
%                         u = - Cc xc - Dc1 y - Dc3 dydf/dt
%                             where dydf/dt is the vector of filtered 
%                             derivatives, using derivative filter
%                             Gf(s)=Nf s/(s+Nf). (default Nf=100)
%
%       OPTIONAL
%        Opt    - options in structure:
%                 Opt.iter   - maximal number of iterations (default: 100)
%                 Opt.eps    - epsilon for the stopping criteria 
%                              (default: eps = 1e-8)
%                 Opt.epsP   - epsilon for the positive definiteness test 
%                              P>=epsP*I (default: epsP = 2.2204e-16)
%                 Opt.X      - Initial Lyapunov matrix for the iteration.
%                              If X=0 then it is calculated 
%                              by lqr command. (default: X = 0)
%                 Opt.x0     - Initial state for the optimization where
%                              Objective=x0'*P*x0. If x0=0 then the
%                              Objective=trace(P). (default x0 = 0) 
%                 Opt.nc     - Order of the dynamic controller. 
%                              i.e.: nc = 3.(default: nc = 2)
%                 Opt.CS     - Controller structure matrix - which 
%                              prescribes the controller structure. 
%                              CS has the size of F and contains 1 or 0. 
%                              i.e.: for ct='p', and for m=l, to obtain  
%                              fully decentralized control CS=eye(m,l).
%                              (default CS=ones(size(F)))
%                 Opt.Nf     - Filter constant for PID, PD, D, DOFID and 
%                              DOFD controller design. Gf(s)=Nf s/(s+Nf). 
%                              (default Nf=100)
%                 Opt.verbose
%                            - Display information. 
%                                0: no information is diplayed,
%                                1: few infomration is displayed,
%                                2: basic infomration is displayed,
%                                3: basic information and controller 
%                                   structure is displayed.
%                              (default verbose=3)
%                 Opt.Ci     - Output matrix for Integral part.
%                              (default Ci=C)
%                 Opt.Cd     - Output matrix for Derivative part.
%                              (default Cd=C)
%                 Opt.mlop   - Maximal number of iterations before stopping
%                              by warning lack of progress.
%                 Opt.settings
%                            - Options structure for Yalmip (sdpsettings). 
%                              The structure is same as for the 
%                              sdpsettings: {'name', value,...}.
%                              i.e.: Opt.settings={'solver','sdpt3'}.
%
%     OUTPUTS:
%        F      - static output-feedback gain matrix,
%        P      - Lyapunov matrix,
%        E      - Closed-loop system eigen values,
%        rv     - return value,
%        dinfo  - design information (struct).
%
%     OTHER INFO:
%       Weighting matrix size (Q,R,N):
%        ct=='p'     - Q(n,n), R(m,m), N(n,m)       
%        ct=='pi'    - Q(n+li,n+li), R(m,m), N(n+li,m) 
%        ct=='pid'   - Q(n+li+ld,n+2*li+ld), R(m,m), N(n+li+ld,m) 
%        ct=='pd'    - Q(n+ld,n+ld), R(m,m), N(n+ld,m)        
%        ct=='d'     - Q(n+ld,n+ld), R(m,m), N(n+ld,m)        
%        ct=='dof'   - Q(n+nc,n+nc), R(m+nc,m+nc), N(n+nc,m+nc)  
%        ct=='dofi'  - Q(n+nc+li,n+nc+li), R(m+nc,m+nc), N(n+nc+li,m+nc)  
%        ct=='dofid' - Q(n+nc+li+ld,n+nc+li+ld), R(m+nc,m+nc), 
%                      N(n+nc+li+ld,m+nc)   
%        ct=='dofd'  - Q(n+nc+ld,n+nc+ld), R(m+nc,m+nc), N(n+nc+ld,m+nc)     
%       where
%             n - number of states,  
%             m - number of inputs,  
%             l - number of outputs, 
%            li - number of outputs for integral part, (default li=l)
%            ld - number of outputs for derival part, (default ld=l)
%            nc - order of the dynamic controller (default nc=2)
%
%     EXAMPLE
%       % Generating a MIMO system with 4 states, 2 inputs and outputs:
%       n=4;m=2;l=2;
%       % ok=-1;
%       % while ok<=0
%       %     sys=rss(n,l,m);
%       %     if any(sys.d(:))==0
%       %         ok=1;
%       %     end
%       % end
%       % Or assume a MIMO system with 4 states, 2 inputs and outputs:
%       A=[ -3.524  -0.5312   0.3998  -0.5916;
%          -0.2418   -2.742  -0.2272  -0.6051;
%          -0.5408  -0.3698   -1.815    1.351;
%          -0.6642   0.2394   -1.429   -1.948];
%       B=[  1.59  -2.115;
%          -2.032       0;
%           2.355  0.8848;
%               0       0];
%       C=[     0        0   0.3742  0.06553;
%          -1.038    1.524    1.239  -0.6803];
%       D=[0 0;0 0];
%       sys=ss(A,B,C,D);
%       % Assume that we want to design a decentralized controller: 
%       %  - PI controller for the first subsystem
%       %  - PID for the second subsystem
%       Opt.Cd=sys.c(2,:);  % Output matrix for the derivative part
%       Opt.CS=[1 0 1 0 0;0 1 0 1 1]; % Structure matrix
%       li=l;   % number of outputs for the integral part
%       ld=1;   % number of outputs for the derivative part 
%       % Defining the weighting matrices
%       Q=1*eye(n+li+ld);
%       R=1*eye(m);
%       N=0.1*ones(n+li+ld,m);
%       % Calling the function
%       [K,S,E]=oflqr(sys,Q,R,N,'pid',Opt)
%       disp('PI Controller:');
%       fprintf('P=%f, I=%f\n',K(1,1),K(1,3));
%       disp('PID Controller:');
%       fprintf('P=%f, I=%f, D=%f\n',K(2,2),K(2,4),K(2,5));
%
%     REQUIREMENTS:
%       Matlab:
%       - Control System Toolbox installed
%       - YALMIP installed (R2015xxx or newer)
%       - LMI solver installed (sdpt3, sedumi, mosek, ...)
%       Octave:
%       - Control package installed and loaded
%       - YALMIP installed (R2015xxx or newer)
%       - LMI solver installed (sdpt3, sedumi, ...)
%
%       Tested with Matlab 2017a/Octave 4.0, YALMIP R20150918. 
%
%  See also LQR DLQR, LQRY, LQI, LQGREG, LQGTRACK, LQG, CARE, DARE.
%
%  For more information please visit: http://adrianilka.eu/oflqrtoolbox.htm
%
%  Version: 1.7.0 (21-10-2018); Author: Adrian Ilka (www.adrianilka.eu)
narginchk(5,6);
%% ERROR MESSAGES
textMI=sprintf('\nFor more info see help OFLQR.');
errormsg{1}=sprintf('Error n.1:\nThe ct need to be one of {''p'',''pi'',''pid'',''pd'',''d'',''dof'',''dofi'',''dofid'',''dofd''}.%s',textMI);    
errormsg{2}=sprintf('Error n.2:\nControl package is not installed or loaded!\n.\nFor more info see help OFLQR');    
errormsg{3}=sprintf('Error n.3:\nSystem should be an object (ss) or cell containing objects (ss) (for polytopic system).%s',textMI);  
errormsg{4}=sprintf('Error n.4:\nControl System Toolbox is not installed or added to the path!.%s',textMI);
errormsg{5}=sprintf('Error n.5:\nCell should have size px1.\nFor more info see help OFLQR');
errormsg{6}=sprintf('Error n.6:\nSystem should be an object (ss) or cell containing objects (ss).%s',textMI');
errormsg{7}=sprintf('Error n.7:\nSystems should be proper.\n%s',textMI);
errormsg{8}=sprintf('Error n.8:\nThe Nf need to be Nf>0.\n%s',textMI);

errormsg{9}=sprintf('Error n.9:\nThe Q matrix has wrong size.\nFor ct=''p'': Q(n,n),  n - number of states.%s',textMI);
errormsg{10}=sprintf('Error n.10:\nThe R matrix has wrong size.\nFor ct=''p'': R(m,m),  m - number of outputs.%s',textMI);
errormsg{11}=sprintf('Error n.11:\nThe N matrix has wrong size.\nFor ct=''p'': N(n,m),\n  n - number of states\n  m - number of inputs%s',textMI);
errormsg{12}=sprintf('Error n.12:\nThe X matrix has wrong size.\nFor ct=''p'': X(n,n),  n - number of states.%s',textMI);
errormsg{13}=sprintf('Error n.13:\nThe X matrix is not positive definite.%s',textMI);
errormsg{14}=sprintf('Error n.14:\nThe CS structure matrix has wrong size.\nFor ct=''p'': CS(m,l),  m - number of inputs,  l - number of outputs.%s',textMI);
errormsg{15}=sprintf('Error n.15:\nThe CS structure matrix must contain only 1 or 0.%s',textMI);

errormsg{16}=sprintf('Error n.16:\nThe Q matrix has wrong size.\nFor ct=''pi'': Q(n+li,n+li), because the system is augmented with the integrals of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n\nThe augmented system:\n  Aaug=[A,zeros(n,li);Ci,zeros(li,li)];\n  Baug=[B;zeros(li,m)];\n  Caug=[C,zeros(l,li);zeros(li,n),eye(li,li)];\nBy default Ci=C, hence li=l.%s',textMI);
errormsg{17}=sprintf('Error n.17:\nThe R matrix has wrong size.\nFor ct=''pi'': R(m,m),  m - number of outputs.%s',textMI);
errormsg{18}=sprintf('Error n.18:\nThe N matrix has wrong size.\nFor ct=''pi'': N(n+li,m), because the system is augmented with the integrals of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n\nThe augmented system:\n  Aaug=[A,zeros(n,li);Ci,zeros(li,li)];\n  Baug=[B;zeros(li,m)];\n  Caug=[C,zeros(l,li);zeros(li,n),eye(li,li)];\nBy default Ci=C, hence li=l.%s',textMI);
errormsg{19}=sprintf('Error n.19:\nThe X matrix has wrong size.\nFor ct=''pi'': X(n+li,n+li), because the system is augmented with the integrals of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n\nThe augmented system:\n  Aaug=[A,zeros(n,li);Ci,zeros(li,li)];\n  Baug=[B;zeros(li,m)];\n  Caug=[C,zeros(l,li);zeros(li,n),eye(li,li)];\nBy default Ci=C, hence li=l.%s',textMI);
errormsg{20}=sprintf('Error n.20:\nThe CS structure matrix has wrong size.\nFor ct=''pi'': CS(m,l+li), because the system is augmented with the integrals of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n\nThe augmented system:\n  Aaug=[A,zeros(n,li);Ci,zeros(li,li)];\n  Baug=[B;zeros(li,m)];\n  Caug=[C,zeros(l,li);zeros(li,n),eye(li,li)];\nBy default Ci=C, hence li=l.%s',textMI);

errormsg{21}=sprintf('Error n.21:\nThe Q matrix has wrong size.\nFor ct=''pid'': Q(n+li+ld,n+li+ld), because the system is augmented with the integrals and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  ld - number of outputs for the derivative part\n\nThe augmented system with filter constant Nf:\nBf=Nf*eye(ld);\nAf=-Nf*eye(ld);\n\nAaug=[A,zeros(n,li),zeros(n,ld);\n      Ci,zeros(li,li),zeros(li,ld);\n      Bf*Cd,zeros(ld,li),Af];\nBaug=[B;zeros(li,m);zeros(ld,m)];\nCaug=[C,zeros(l,li),zeros(l,ld);\n      zeros(li,n),eye(li,li),zeros(li,ld);\n      Bf*Cd,zeros(ld,li),Af];\n%s',textMI);
errormsg{22}=sprintf('Error n.22:\nThe R matrix has wrong size.\nFor ct=''pid'': R(m,m),  m - number of outputs.%s',textMI);
errormsg{23}=sprintf('Error n.23:\nThe N matrix has wrong size.\nFor ct=''pid'': N(n+li+ld,m), because the system is augmented with the integrals and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  ld - number of outputs for the derivative part\n\nThe augmented system with filter constant Nf:\nBf=Nf*eye(ld);\nAf=-Nf*eye(ld);\n\nAaug=[A,zeros(n,li),zeros(n,ld);\n      Ci,zeros(li,li),zeros(li,ld);\n      Bf*Cd,zeros(ld,li),Af];\nBaug=[B;zeros(li,m);zeros(ld,m)];\nCaug=[C,zeros(l,li),zeros(l,ld);\n      zeros(li,n),eye(li,li),zeros(li,ld);\n      Bf*Cd,zeros(ld,li),Af];\n%s',textMI);
errormsg{24}=sprintf('Error n.24:\nThe X matrix has wrong size.\nFor ct=''pid'': X(n+li+ld,n+li+ld), because the system is augmented with the integrals and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  ld - number of outputs for the derivative part\n\nThe augmented system with filter constant Nf:\nBf=Nf*eye(ld);\nAf=-Nf*eye(ld);\n\nAaug=[A,zeros(n,li),zeros(n,ld);\n      Ci,zeros(li,li),zeros(li,ld);\n      Bf*Cd,zeros(ld,li),Af];\nBaug=[B;zeros(li,m);zeros(ld,m)];\nCaug=[C,zeros(l,li),zeros(l,ld);\n      zeros(li,n),eye(li,li),zeros(li,ld);\n      Bf*Cd,zeros(ld,li),Af];\n%s',textMI);
errormsg{25}=sprintf('Error n.25:\nThe CS structure matrix has wrong size.\nFor ct=''pid'': CS(m,l+li+ld), because the system is augmented with the integrals and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  ld - number of outputs for the derivative part\n\nThe augmented system with filter constant Nf:\nBf=Nf*eye(ld);\nAf=-Nf*eye(ld);\n\nAaug=[A,zeros(n,li),zeros(n,ld);\n      Ci,zeros(li,li),zeros(li,ld);\n      Bf*Cd,zeros(ld,li),Af];\nBaug=[B;zeros(li,m);zeros(ld,m)];\nCaug=[C,zeros(l,li),zeros(l,ld);\n      zeros(li,n),eye(li,li),zeros(li,ld);\n      Bf*Cd,zeros(ld,li),Af];\n%s',textMI);

errormsg{26}=sprintf('Error n.26:\nThe Q matrix has wrong size.\nFor ct=''pd'': Q(n+ld,n+ld), because the system is augmented with filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  ld - number of outputs for the derivative part\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,ld);Bf*Cd,Af];\n  Baug=[B;zeros(ld,m)];\n  Caug=[C,zeros(l,ld);Bf*Cd,Af];\n%s',textMI);
errormsg{27}=sprintf('Error n.17:\nThe R matrix has wrong size.\nFor ct=''pd'': R(m,m),  m - number of outputs.%s',textMI);
errormsg{28}=sprintf('Error n.28:\nThe N matrix has wrong size.\nFor ct=''pd'': N(n+ld,m), because the system is augmented with the filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  ld - number of outputs for the derivative part\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,ld);Bf*Cd,Af];\n  Baug=[B;zeros(ld,m)];\n  Caug=[C,zeros(l,ld);Bf*Cd,Af];\n%s',textMI);
errormsg{29}=sprintf('Error n.29:\nThe X matrix has wrong size.\nFor ct=''pd'': X(n+ld,n+ld), because the system is augmented with filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  ld - number of outputs for the derivative part\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,ld);Bf*Cd,Af];\n  Baug=[B;zeros(ld,m)];\n  Caug=[C,zeros(l,ld);Bf*Cd,Af];\n%s',textMI);
errormsg{30}=sprintf('Error n.30:\nThe CS structure matrix has wrong size.\nFor ct=''pd'': CS(m,l+ld), because the system is augmented with filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  ld - number of outputs for the derivative part\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,ld);Bf*Cd,Af];\n  Baug=[B;zeros(ld,m)];\n  Caug=[C,zeros(l,ld);Bf*Cd,Af];\n%s',textMI);

errormsg{31}=sprintf('Error n.31:\nThe Q matrix has wrong size.\nFor ct=''d'': Q(n+ld,n+ld), because the system is augmented with the filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  ld - number of outputs for the derivative part\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,ld);Bf*Cd,Af];\n  Baug=[B;zeros(ld,m)];\n  Caug=[C,zeros(l,ld);Bf*Cd,Af];\n%s',textMI);
errormsg{32}=sprintf('Error n.32:\nThe R matrix has wrong size.\nFor ct=''d'': R(m,m),  m - number of outputs.%s',textMI);
errormsg{33}=sprintf('Error n.33:\nThe N matrix has wrong size.\nFor ct=''d'': N(n+ld,m), because the system is augmented with the filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  ld - number of outputs for the derivative part\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,ld);Bf*Cd,Af];\n  Baug=[B;zeros(ld,m)];\n  Caug=[C,zeros(l,ld);Bf*Cd,Af];\n%s',textMI);
errormsg{34}=sprintf('Error n.34:\nThe X matrix has wrong size.\nFor ct=''d'': X(n+ld,n+ld), because the system is augmented with the filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  ld - number of outputs for the derivative part\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,ld);Bf*Cd,Af];\n  Baug=[B;zeros(ld,m)];\n  Caug=[C,zeros(l,ld);Bf*Cd,Af];\n%s',textMI);
errormsg{35}=sprintf('Error n.35:\nThe CS structure matrix has wrong size.\nFor ct=''d'': CS(m,ld), because the system is augmented with the filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  ld - number of outputs for the derivative part\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,ld);Bf*Cd,Af];\n  Baug=[B;zeros(ld,m)];\n  Caug=[C,zeros(l,ld);Bf*Cd,Af];\n%s',textMI);

errormsg{36}=sprintf('Error n.36:\nThe Q matrix has wrong size.\nFor ct=''dof'': Q(n+nc,n+nc), because the system is augmented with the dynamic controller''s states.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  nc - order of the dynamic controller\n\nThe augmented system:\n  Aaug=[A,zeros(n,nc);zeros(nc,n),zeros(nc,nc)];\n  Baug=[B,zeros(n,nc);zeros(nc,m),eye(nc,nc)];\n  Caug=[C,zeros(l,nc);zeros(nc,n),eye(nc,nc)];\n%s',textMI);
errormsg{37}=sprintf('Error n.37:\nThe R matrix has wrong size.\nFor ct=''dof'': R(m+nc,m+nc), because the system is augmented with the dynamic controller''s states.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  nc - order of the dynamic controller\n\nThe augmented system:\n  Aaug=[A,zeros(n,nc);zeros(nc,n),zeros(nc,nc)];\n  Baug=[B,zeros(n,nc);zeros(nc,m),eye(nc,nc)];\n  Caug=[C,zeros(l,nc);zeros(nc,n),eye(nc,nc)];\n%s',textMI);
errormsg{38}=sprintf('Error n.38:\nThe N matrix has wrong size.\nFor ct=''dof'': N(n+nc,m+nc), because the system is augmented with the dynamic controller''s states.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  nc - order of the dynamic controller\n\nThe augmented system:\n  Aaug=[A,zeros(n,nc);zeros(nc,n),zeros(nc,nc)];\n  Baug=[B,zeros(n,nc);zeros(nc,m),eye(nc,nc)];\n  Caug=[C,zeros(l,nc);zeros(nc,n),eye(nc,nc)];\n%s',textMI);
errormsg{39}=sprintf('Error n.39:\nThe X matrix has wrong size.\nFor ct=''dof'': X(n+nc,n+nc), because the system is augmented with the dynamic controller''s states.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  nc - order of the dynamic controller\n\nThe augmented system:\n  Aaug=[A,zeros(n,nc);zeros(nc,n),zeros(nc,nc)];\n  Baug=[B,zeros(n,nc);zeros(nc,m),eye(nc,nc)];\n  Caug=[C,zeros(l,nc);zeros(nc,n),eye(nc,nc)];\n%s',textMI);
errormsg{40}=sprintf('Error n.40:\nThe CS structure matrix has wrong size.\nFor ct=''dof'': CS(m+nc,l+nc), because the system is augmented with the dynamic controller''s states.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  nc - order of the dynamic controller\n\nThe augmented system:\n  Aaug=[A,zeros(n,nc);zeros(nc,n),zeros(nc,nc)];\n  Baug=[B,zeros(n,nc);zeros(nc,m),eye(nc,nc)];\n  Caug=[C,zeros(l,nc);zeros(nc,n),eye(nc,nc)];\n%s',textMI);

errormsg{41}=sprintf('Error n.41:\nThe Q matrix has wrong size.\nFor ct=''dofi'': Q(n+nc+li,n+nc+li), because the system is augmented with the dynamic controller''s states and integrals of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  nc - order of the dynamic controller\n\nThe augmented system:\n  Aaug=[A,zeros(n,nc),zeros(n,li);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,li);\n        Ci,zeros(li,nc),zeros(li,li)];\n  Baug=[B,zeros(n,nc);zeros(nc,m),eye(nc,nc);zeros(li,m),zeros(li,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,li);\n        zeros(nc,n),eye(nc,nc),zeros(nc,li);\n        zeros(li,n),zeros(li,nc),eye(li,li)];\n%s',textMI);
errormsg{42}=sprintf('Error n.42:\nThe R matrix has wrong size.\nFor ct=''dofi'': R(m+nc,m+nc), because the system is augmented with the dynamic controller''s states and integrals of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  nc - order of the dynamic controller\n\nThe augmented system:\n  Aaug=[A,zeros(n,nc),zeros(n,li);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,li);\n        Ci,zeros(li,nc),zeros(li,li)];\n  Baug=[B,zeros(n,nc);zeros(nc,m),eye(nc,nc);zeros(li,m),zeros(li,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,li);\n        zeros(nc,n),eye(nc,nc),zeros(nc,li);\n        zeros(li,n),zeros(li,nc),eye(li,li)];\n%s',textMI);
errormsg{43}=sprintf('Error n.43:\nThe N matrix has wrong size.\nFor ct=''dofi'': N(n+nc+li,m+nc), because the system is augmented with the dynamic controller''s states and integrals of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  nc - order of the dynamic controller\n\nThe augmented system:\n  Aaug=[A,zeros(n,nc),zeros(n,li);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,li);\n        Ci,zeros(li,nc),zeros(li,li)];\n  Baug=[B,zeros(n,nc);zeros(nc,m),eye(nc,nc);zeros(li,m),zeros(li,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,li);\n        zeros(nc,n),eye(nc,nc),zeros(nc,li);\n        zeros(li,n),zeros(li,nc),eye(li,li)];\n%s',textMI);
errormsg{44}=sprintf('Error n.44:\nThe X matrix has wrong size.\nFor ct=''dofi'': X(n+nc+li,n+nc+li), because the system is augmented with the dynamic controller''s states and integrals of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  nc - order of the dynamic controller\n\nThe augmented system:\n  Aaug=[A,zeros(n,nc),zeros(n,li);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,li);\n        Ci,zeros(li,nc),zeros(li,li)];\n  Baug=[B,zeros(n,nc);zeros(nc,m),eye(nc,nc);zeros(li,m),zeros(li,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,li);\n        zeros(nc,n),eye(nc,nc),zeros(nc,li);\n        zeros(li,n),zeros(li,nc),eye(li,li)];\n%s',textMI);
errormsg{45}=sprintf('Error n.45:\nThe CS structure matrix has wrong size.\nFor ct=''dofi'': CS(m+nc,l+li+nc), because the system is augmented with the dynamic controller''s states and integrals of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  nc - order of the dynamic controller\n\nThe augmented system:\n  Aaug=[A,zeros(n,nc),zeros(n,li);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,li);\n        Ci,zeros(li,nc),zeros(li,li)];\n  Baug=[B,zeros(n,nc);zeros(nc,m),eye(nc,nc);zeros(li,m),zeros(li,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,li);\n        zeros(nc,n),eye(nc,nc),zeros(nc,li);\n        zeros(li,n),zeros(li,nc),eye(li,li)];\n%s',textMI);

errormsg{46}=sprintf('Error n.46:\nThe Q matrix has wrong size.\nFor ct=''dofid'': Q(n+nc+li+ld,n+nc+li+ld), because the system is augmented with the dynamic controller''s states, integrals and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  ld - number of outputs for the derivative part\n  nc - order of the dynamic controller\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,nc),zeros(n,li),zeros(n,ld);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,li),zeros(nc,ld);\n        Ci,zeros(li,nc),zeros(li,li),zeros(li,ld);\n        Bf*Cd,zeros(ld,nc),zeros(ld,li),Af];\n  Baug=[B,zeros(n,nc);\n        zeros(nc,m),eye(nc,nc);\n        zeros(li,m),zeros(li,nc);\n        zeros(ld,m),zeros(ld,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,li),zeros(l,ld);\n        zeros(nc,n),eye(nc,nc),zeros(nc,li),zeros(nc,ld);\n        zeros(li,n),zeros(li,nc),eye(li,li),zeros(li,ld);\n        Bf*Cd{j},zeros(ld,nc),zeros(ld,li),Af];\n%s',textMI);
errormsg{47}=sprintf('Error n.47:\nThe R matrix has wrong size.\nFor ct=''dofid'': R(m+nc,m+nc), because the system is augmented with the dynamic controller''s states, integrals and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  ld - number of outputs for the derivative part\n  nc - order of the dynamic controller\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,nc),zeros(n,li),zeros(n,ld);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,li),zeros(nc,ld);\n        Ci,zeros(li,nc),zeros(li,li),zeros(li,ld);\n        Bf*Cd,zeros(ld,nc),zeros(ld,li),Af];\n  Baug=[B,zeros(n,nc);\n        zeros(nc,m),eye(nc,nc);\n        zeros(li,m),zeros(li,nc);\n        zeros(ld,m),zeros(ld,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,li),zeros(l,ld);\n        zeros(nc,n),eye(nc,nc),zeros(nc,li),zeros(nc,ld);\n        zeros(li,n),zeros(li,nc),eye(li,li),zeros(li,ld);\n        Bf*Cd{j},zeros(ld,nc),zeros(ld,li),Af];\n%s',textMI);
errormsg{48}=sprintf('Error n.48:\nThe N matrix has wrong size.\nFor ct=''dofid'': N(n+nc+li+ld,m+nc), because the system is augmented with the dynamic controller''s states, integrals and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  ld - number of outputs for the derivative part\n  nc - order of the dynamic controller\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,nc),zeros(n,li),zeros(n,ld);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,li),zeros(nc,ld);\n        Ci,zeros(li,nc),zeros(li,li),zeros(li,ld);\n        Bf*Cd,zeros(ld,nc),zeros(ld,li),Af];\n  Baug=[B,zeros(n,nc);\n        zeros(nc,m),eye(nc,nc);\n        zeros(li,m),zeros(li,nc);\n        zeros(ld,m),zeros(ld,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,li),zeros(l,ld);\n        zeros(nc,n),eye(nc,nc),zeros(nc,li),zeros(nc,ld);\n        zeros(li,n),zeros(li,nc),eye(li,li),zeros(li,ld);\n        Bf*Cd{j},zeros(ld,nc),zeros(ld,li),Af];\n%s',textMI);
errormsg{49}=sprintf('Error n.49:\nThe X matrix has wrong size.\nFor ct=''dofid'': X(n+nc+li+ld,n+nc+li+ld), because the system is augmented with the dynamic controller''s states, integrals and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  ld - number of outputs for the derivative part\n  nc - order of the dynamic controller\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,nc),zeros(n,li),zeros(n,ld);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,li),zeros(nc,ld);\n        Ci,zeros(li,nc),zeros(li,li),zeros(li,ld);\n        Bf*Cd,zeros(ld,nc),zeros(ld,li),Af];\n  Baug=[B,zeros(n,nc);\n        zeros(nc,m),eye(nc,nc);\n        zeros(li,m),zeros(li,nc);\n        zeros(ld,m),zeros(ld,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,li),zeros(l,ld);\n        zeros(nc,n),eye(nc,nc),zeros(nc,li),zeros(nc,ld);\n        zeros(li,n),zeros(li,nc),eye(li,li),zeros(li,ld);\n        Bf*Cd{j},zeros(ld,nc),zeros(ld,li),Af];\n%s',textMI);
errormsg{50}=sprintf('Error n.50:\nThe CS structure matrix has wrong size.\nFor ct=''dofid'': CS(m+nc,l+nc+li+ld), because the system is augmented with the dynamic controller''s states, integrals and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  li - number of outputs for the integral part\n  ld - number of outputs for the derivative part\n  nc - order of the dynamic controller\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,nc),zeros(n,li),zeros(n,ld);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,li),zeros(nc,ld);\n        Ci,zeros(li,nc),zeros(li,li),zeros(li,ld);\n        Bf*Cd,zeros(ld,nc),zeros(ld,li),Af];\n  Baug=[B,zeros(n,nc);\n        zeros(nc,m),eye(nc,nc);\n        zeros(li,m),zeros(li,nc);\n        zeros(ld,m),zeros(ld,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,li),zeros(l,ld);\n        zeros(nc,n),eye(nc,nc),zeros(nc,li),zeros(nc,ld);\n        zeros(li,n),zeros(li,nc),eye(li,li),zeros(li,ld);\n        Bf*Cd{j},zeros(ld,nc),zeros(ld,li),Af];\n%s',textMI);

errormsg{51}=sprintf('Error n.51:\nThe Q matrix has wrong size.\nFor ct=''dofd'': Q(n+nc+ld,n+nc+ld), because the system is augmented with the dynamic controller''s states and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  ld - number of outputs for the derivative part\n  nc - order of the dynamic controller\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,nc),zeros(n,ld);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,ld);\n        Bf*Cd,zeros(ld,nc),Af];\n  Baug=[B,zeros(n,nc);\n        zeros(nc,m),eye(nc,nc);\n        zeros(ld,m),zeros(ld,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,ld);\n        zeros(nc,n),eye(nc,nc),zeros(nc,ld);\n        Bf*Cd,zeros(ld,nc),Af];\n%s',textMI);
errormsg{52}=sprintf('Error n.52:\nThe R matrix has wrong size.\nFor ct=''dofd'': R(m+nc,m+nc), because the system is augmented with the dynamic controller''s states and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  ld - number of outputs for the derivative part\n  nc - order of the dynamic controller\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,nc),zeros(n,ld);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,ld);\n        Bf*Cd,zeros(ld,nc),Af];\n  Baug=[B,zeros(n,nc);\n        zeros(nc,m),eye(nc,nc);\n        zeros(ld,m),zeros(ld,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,ld);\n        zeros(nc,n),eye(nc,nc),zeros(nc,ld);\n        Bf*Cd,zeros(ld,nc),Af];\n%s',textMI);
errormsg{53}=sprintf('Error n.53:\nThe N matrix has wrong size.\nFor ct=''dofd'': N(n+nc+ld,m+nc), because the system is augmented with the dynamic controller''s states and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  ld - number of outputs for the derivative part\n  nc - order of the dynamic controller\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,nc),zeros(n,ld);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,ld);\n        Bf*Cd,zeros(ld,nc),Af];\n  Baug=[B,zeros(n,nc);\n        zeros(nc,m),eye(nc,nc);\n        zeros(ld,m),zeros(ld,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,ld);\n        zeros(nc,n),eye(nc,nc),zeros(nc,ld);\n        Bf*Cd,zeros(ld,nc),Af];\n%s',textMI);
errormsg{54}=sprintf('Error n.54:\nThe X matrix has wrong size.\nFor ct=''dofd'': X(n+nc+ld,n+nc+ld), because the system is augmented with the dynamic controller''s states and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  ld - number of outputs for the derivative part\n  nc - order of the dynamic controller\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,nc),zeros(n,ld);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,ld);\n        Bf*Cd,zeros(ld,nc),Af];\n  Baug=[B,zeros(n,nc);\n        zeros(nc,m),eye(nc,nc);\n        zeros(ld,m),zeros(ld,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,ld);\n        zeros(nc,n),eye(nc,nc),zeros(nc,ld);\n        Bf*Cd,zeros(ld,nc),Af];\n%s',textMI);
errormsg{55}=sprintf('Error n.55:\nThe CS structure matrix has wrong size.\nFor ct=''dofd'': CS(m+nc,l+nc+ld), because the system is augmented with the dynamic controller''s states and filtered derivatives of outputs.\n  n - number of states\n  m - number of inputs\n  l - number of outputs\n  ld - number of outputs for the derivative part\n  nc - order of the dynamic controller\n\nThe augmented system with filter constant Nf:\n  Bf=Nf*eye(ld);\n  Af=-Nf*eye(ld);\n\n  Aaug=[A,zeros(n,nc),zeros(n,ld);\n        zeros(nc,n),zeros(nc,nc),zeros(nc,ld);\n        Bf*Cd,zeros(ld,nc),Af];\n  Baug=[B,zeros(n,nc);\n        zeros(nc,m),eye(nc,nc);\n        zeros(ld,m),zeros(ld,nc)];\n  Caug=[C,zeros(l,nc),zeros(l,ld);\n        zeros(nc,n),eye(nc,nc),zeros(nc,ld);\n        Bf*Cd,zeros(ld,nc),Af];\n%s',textMI);

errormsg{56}=sprintf('Error n.56:\nThere is problem with the Q matrix.%s',textMI);
errormsg{57}=sprintf('Error n.57:\nThere is problem with the R matrix.%s',textMI);
errormsg{58}=sprintf('Error n.58:\nThere is problem with the N matrix.%s',textMI);
errormsg{59}=sprintf('Error n.59:\nThere is problem with the X matrix.%s',textMI);
errormsg{60}=sprintf('Error n.60:\nThere is problem with the CS matrix.%s',textMI);

errormsg{61}=sprintf('Error n.61:\nThis approach allows only sys.d=0.%s',textMI);
errormsg{62}=sprintf('Error n.62:\nThe Q-NR^-1N^T need to be positive semidefinite.%s',textMI);
errormsg{63}=sprintf('Error n.63:\nThe R need to be positive definite (R>0).%s',textMI);
errormsg{64}=sprintf('Error n.64:\nThe eps need to be eps>0.%s',textMI);
errormsg{65}=sprintf('Error n.65:\nYalmip is not installed or added to the path! (https://yalmip.github.io/)%s',textMI);

errormsg{66}=sprintf('Error n.66:\nThe number of Ci{i} matrices in the struct Ci should be equal to the number of polytops.%s',textMI);
errormsg{67}=sprintf('Error n.67:\nThe size of the Ci matix/matrices should be (li x n),\n n - number of states\n li - number of outputs for the integral part%s',textMI);
errormsg{68}=sprintf('Error n.68:\nAll the Ci matrices in the cell should have the same size.%s',textMI);
errormsg{69}=sprintf('Error n.69:\nThe Ci should be a matrix or a cell containing matrices.%s',textMI);

errormsg{70}=sprintf('Error n.70:\nThe number of Cd{i} matrices in the struct Ci should be equal to the number of polytops.%s',textMI);
errormsg{71}=sprintf('Error n.71:\nThe size of the Cd matrix/matrices should be (ld x n),\n n - number of states\n ld - number of outputs for the derivativ part%s',textMI);
errormsg{72}=sprintf('Error n.72:\nAll the Cd matrices in the cell should have the same size.%s',textMI);
errormsg{73}=sprintf('Error n.73:\nThe Cd should be a matrix or a cell containing matrices.%s',textMI);

errormsg{74}=sprintf('Error n.74:\nThe proposed approach allows only continuous-time ss models.%s',textMI);
errormsg{75}=sprintf('Error n.75:\nThe proposed approach allows only verbose>=0.%s',textMI);
errormsg{76}=sprintf('Error n.76:\nThe proposed approach allows only mlop>1.%s',textMI);
errormsg{77}=sprintf('Error n.77:\nNumber of iterations should be a least iter>=1.%s',textMI);

%% TESTING THE INPUT VARIABLES
% Testing the controller type
cts={'p','pi','pid','pd','d','dof','dofi','dofid','dofd'};
if not(any(strcmp(cts, ct)))
    error(errormsg{1});
end
% Testing the system
isOctave = exist('OCTAVE_VERSION', 'builtin');
if isOctave~=0
    % Octave part
    if license('test', 'control')~=1
        error(errormsg{2});
    end
    
    if iscell(sys)
        p=length(sys);

        As=cell(1,p);
        Bs=cell(1,p);
        Cs=cell(1,p);
        Ds=cell(1,p);
        for j=1:p
            As{j}=sys{j}.a;
            Bs{j}=sys{j}.b;
            Cs{j}=sys{j}.c;
            Ds{j}=sys{j}.d;
            if sys{j}.ts~=0
                error(errormsg{74});
            end
        end
    elseif isobject(sys)
        p=1;
        As{1}=sys.a;
        Bs{1}=sys.b;
        Cs{1}=sys.c;
        Ds{1}=sys.d;
        if sys.ts~=0
            error(errormsg{74});
        end
    else
        error(errormsg{3});
    end
else
    % Matlab part
    if license('test', 'control_toolbox')~=1
        error(errormsg{4});
    end
    
    if isobject(sys)
        [~,~,p,p2]=size(sys);
        if p2~=1
            error(errormsg{5});
        end
        
        As=cell(1,p);
        Bs=cell(1,p);
        Cs=cell(1,p);
        Ds=cell(1,p);

        for j=1:p
            As{j}=sys(:,:,j).a;
            Bs{j}=sys(:,:,j).b;
            Cs{j}=sys(:,:,j).c;
            Ds{j}=sys(:,:,j).d;
            if sys(:,:,j).ts~=0
                error(errormsg{74});
            end
        end
    elseif iscell(sys)
        p=length(sys);
        
        As=cell(1,p);
        Bs=cell(1,p);
        Cs=cell(1,p);
        Ds=cell(1,p);

        for j=1:p
            As{j}=sys{j}.a;
            Bs{j}=sys{j}.b;
            Cs{j}=sys{j}.c;
            Ds{j}=sys{j}.d;
            if sys{j}.ts~=0
                error(errormsg{74});
            end
        end
    else
        error(errormsg{6});
    end
end

Anm=cell(1,p);
Bnm=cell(1,p);
Cnm=cell(1,p);
Dnm=cell(1,p);

for j=1:p
    Anm{j}=size(As{j});
    Bnm{j}=size(Bs{j});
    Cnm{j}=size(Cs{j});
    Dnm{j}=size(Ds{j});

    if (Anm{j}(1)~=Anm{j}(2))||(Anm{j}(1)~=Bnm{j}(1))||(Anm{j}(1)~=Cnm{j}(2))||(Cnm{j}(1)~=Dnm{j}(1))||(Bnm{j}(2)~=Dnm{j}(2))
        error(errormsg{7});
    end
    if (Anm{1}(1)~=Anm{j}(1))||(Bnm{1}(2)~=Bnm{j}(2))||(Cnm{1}(1)~=Cnm{j}(1))
        error(errormsg{7});
    end
end

% Determining the size of the matrices 
[ns,ms]=size(Bs{1});
[ls,~]=size(Cs{1});

Qnm=size(Q);
Rnm=size(R);
Nnm=size(N);

% Defining the default parameters
default_iter=100;
default_eps=1e-8;
default_X=0;
default_nc=2;
default_CS=1;
default_settings={};
default_epsP=eps;
default_Nf=100;
default_Ci=Cs;
default_Cd=Cs;
default_verbose=3;
default_mlop=5;
default_x0=0;
% Checking the varargin
if nargin==6
    options=varargin{1};

    if not(isfield(options,'iter'))
        itern=default_iter;
    else
        itern=options.iter;
        validateattributes(itern,{'double'},{'nonempty'},'OFLQR','Options.iter');
        if itern<1
            error(errormsg{77});
        end
    end

    if not(isfield(options,'eps'))
        eps_s=default_eps;
    else
        eps_s=options.eps;
        validateattributes(eps_s,{'numeric'},{'nonempty'},'OFLQR','Options.eps');
    end

    if not(isfield(options,'X'))
        X=default_X;
    else
        X=options.X;
        validateattributes(X,{'numeric'},{'nonempty'},'OFLQR','Options.X');
    end
    
    if not(isfield(options,'nc'))
        nc=default_nc;
    else
        nc=options.nc;
        validateattributes(nc,{'numeric'},{'nonempty'},'OFLQR','Options.nc');
    end
    
    if not(isfield(options,'CS'))
        CS=default_CS;
    else
        CS=options.CS;
        validateattributes(CS,{'numeric'},{'nonempty'},'OFLQR','Options.CS');
    end
    
    if not(isfield(options,'x0'))
        x0=default_x0;
    else
        x0=options.x0;               
    end
    
    if not(isfield(options,'settings'))
        solverOpt=default_settings;
    else
        solverOpt=options.settings;
        validateattributes(solverOpt,{'cell'},{'nonempty'},'OFLQR','Options.settings');
    end
    
    if not(isfield(options,'verbose'))
        verbose=default_verbose;
    else
        verbose=options.verbose;
        validateattributes(verbose,{'numeric'},{'nonempty'},'OFLQR','Options.verbose');
        if (verbose<0)
            error(errormsg{75});
        end
    end
    
    if not(isfield(options,'mlop'))
        mlop=default_mlop;
    else
        mlop=options.mlop;
        validateattributes(mlop,{'numeric'},{'nonempty'},'OFLQR','Options.mlop');
        if (mlop<2)
            error(errormsg{76});
        end
    end
    
    if not(isfield(options,'epsP'))
        epsP=default_epsP;
    else
        epsP=options.epsP;
        validateattributes(epsP,{'numeric'},{'nonempty'},'OFLQR','Options.epsP');
    end
    
    if not(isfield(options,'Nf'))
        Nf=default_Nf;
    else
        Nf=options.Nf;
        validateattributes(Nf,{'numeric'},{'nonempty'},'OFLQR','Options.Nf');
        if Nf<=0
            error(errormsg{8});
        end
    end
    
    if not(isfield(options,'Ci'))
        Ci=default_Ci;
    else        
        Cinm=cell(1,p);
        Ci=cell(1,p);
        if isnumeric(options.Ci)
            validateattributes(options.Ci,{'numeric'},{'nonempty'},'OFLQR','Options.Ci');
            Cinm{1}=size(options.Ci);
            if (Anm{1}(1)~=Cinm{1}(2))
                error(errormsg{67});
            end
            for jc=1:p
                Ci{jc}=options.Ci;
            end            
        elseif iscell(options.Ci)
            lp=length(options.Ci);
            if lp~=p
                error(errormsg{66});
            end            
            for j=1:p
                validateattributes(options.Ci{j},{'numeric'},{'nonempty'},'OFLQR','Options.Ci');
                Cinm{j}=size(options.Ci{j});
                if (Anm{j}(1)~=Cinm{j}(2))
                    error(errormsg{67});
                end
                if (Cinm{1}(1)~=Cinm{j}(1))||(Cinm{1}(2)~=Cinm{j}(2))
                    error(errormsg{68});
                end                
            end
            Ci=options.Ci;
        else
            error(errormsg{69});
        end                    
    end
    if not(isfield(options,'Cd'))
        Cd=default_Cd;
    else
        Cdnm=cell(1,p);   
        Cd=cell(1,p);
        if isnumeric(options.Cd)
            validateattributes(options.Cd,{'numeric'},{'nonempty'},'OFLQR','Options.Cd');
            Cdnm{1}=size(options.Cd);
            if (Anm{1}(1)~=Cdnm{1}(2))
                error(errormsg{71});
            end
            for jc=1:p
                Cd{jc}=options.Cd;
            end
        elseif iscell(options.Cd)
            lp=length(options.Cd);
            if lp~=p
                error(errormsg{70});
            end            
            for j=1:p
                validateattributes(options.Cd{j},{'numeric'},{'nonempty'},'OFLQR','Options.Cd');
                Cdnm{j}=size(options.Cd{j});
                if (Anm{j}(1)~=Cdnm{j}(2))
                    error(errormsg{71});
                end
                if (Cdnm{1}(1)~=Cdnm{j}(1))||(Cdnm{1}(2)~=Cdnm{j}(2))
                    error(errormsg{72});
                end                
            end
            Cd=options.Cd;
        else
            error(errormsg{73});
        end
    end
elseif nargin==5
    itern=default_iter;
    eps_s=default_eps;
    X=default_X;
    nc=default_nc;
    CS=default_CS;
    solverOpt=default_settings;
    epsP=default_epsP;
    Nf=default_Nf;
    Ci=default_Ci;
    Cd=default_Cd;    
    verbose=default_verbose;
    mlop=default_mlop;
    x0=default_x0;
end

%% Augmenting the system based on ct
[li,~]=size(Ci{1});
[ld,~]=size(Cd{1});

Bf=Nf*eye(ld);
Af=-Nf*eye(ld);

A=cell(1,p);
B=cell(1,p);
C=cell(1,p);
if strcmp(ct,'p')
    A=As;
    B=Bs;
    C=Cs;
elseif strcmp(ct,'pi')
    for j=1:p
        A{j}=[As{j},zeros(ns,li);Ci{j},zeros(li,li)];
        B{j}=[Bs{j};zeros(li,ms)];
        C{j}=[Cs{j},zeros(ls,li);zeros(li,ns),eye(li,li)];
    end
elseif strcmp(ct,'pid')
    for j=1:p
        A{j}=[As{j},zeros(ns,li),zeros(ns,ld);
           Ci{j},zeros(li,li),zeros(li,ld);
           Bf*Cd{j},zeros(ld,li),Af];
        B{j}=[Bs{j};zeros(li,ms);zeros(ld,ms)];
        C{j}=[Cs{j},zeros(ls,li),zeros(ls,ld);
           zeros(li,ns),eye(li,li),zeros(li,ld);
           Bf*Cd{j},zeros(ld,li),Af];
    end
elseif strcmp(ct,'pd') 
    for j=1:p
        A{j}=[As{j},zeros(ns,ld);
              Bf*Cd{j},Af];
        B{j}=[Bs{j};zeros(ld,ms)];
        C{j}=[Cs{j},zeros(ls,ld);
              Bf*Cd{j},Af];
    end
elseif strcmp(ct,'d')
    for j=1:p
        A{j}=[As{j},zeros(ns,ls);
              Bf*Cs{j},Af];
        B{j}=[Bs{j};zeros(ls,ms)];
        C{j}=[Bf*Cs{j},Af];
    end
elseif strcmp(ct,'dof')
    for j=1:p
        A{j}=[As{j},zeros(ns,nc);zeros(nc,ns),zeros(nc,nc)];
        B{j}=[Bs{j},zeros(ns,nc);zeros(nc,ms),eye(nc,nc)];
        C{j}=[Cs{j},zeros(ls,nc);zeros(nc,ns),eye(nc,nc)];
    end
elseif strcmp(ct,'dofi')
    for j=1:p
        A{j}=[As{j},zeros(ns,nc),zeros(ns,li);
              zeros(nc,ns),zeros(nc,nc),zeros(nc,li);
              Ci{j},zeros(li,nc),zeros(li,li)];
        B{j}=[Bs{j},zeros(ns,nc);zeros(nc,ms),eye(nc,nc);zeros(li,ms),zeros(li,nc)];
        C{j}=[Cs{j},zeros(ls,nc),zeros(ls,li);
              zeros(nc,ns),eye(nc,nc),zeros(nc,li);
              zeros(li,ns),zeros(li,nc),eye(li,li)];
    end
elseif strcmp(ct,'dofid')
    for j=1:p
        A{j}=[As{j},zeros(ns,nc),zeros(ns,li),zeros(ns,ld);
              zeros(nc,ns),zeros(nc,nc),zeros(nc,li),zeros(nc,ld);
              Ci{j},zeros(li,nc),zeros(li,li),zeros(li,ld);
              Bf*Cd{j},zeros(ld,nc),zeros(ld,li),Af];
        B{j}=[Bs{j},zeros(ns,nc);
              zeros(nc,ms),eye(nc,nc);
              zeros(li,ms),zeros(li,nc);
              zeros(ld,ms),zeros(ld,nc)];
        C{j}=[Cs{j},zeros(ls,nc),zeros(ls,li),zeros(ls,ld);
              zeros(nc,ns),eye(nc,nc),zeros(nc,li),zeros(nc,ld);
              zeros(li,ns),zeros(li,nc),eye(li,li),zeros(li,ld);
              Bf*Cd{j},zeros(ld,nc),zeros(ld,li),Af];
    end
elseif strcmp(ct,'dofd')
    for j=1:p
        A{j}=[As{j},zeros(ns,nc),zeros(ns,ld);
              zeros(nc,ns),zeros(nc,nc),zeros(nc,ld);
              Bf*Cd{j},zeros(ld,nc),Af];
        B{j}=[Bs{j},zeros(ns,nc);
              zeros(nc,ms),eye(nc,nc);
              zeros(ld,ms),zeros(ld,nc)];
        C{j}=[Cs{j},zeros(ls,nc),zeros(ls,ld);
              zeros(nc,ns),eye(nc,nc),zeros(nc,ld);
              Bf*Cd{j},zeros(ld,nc),Af];
    end
end

[n,m]=size(B{1});
[l,~]=size(C{1});
%% TESTING OTHER DEPENDENCES
if any(x0)    
    validateattributes(x0,{'numeric'},{'nrows',n},'OFLQR','Options.x0');        
end
Xnm=size(X);
CSnm=size(CS);

WM{1}=(Qnm(1)~=n)||(Qnm(2)~=n);
WM{2}=(Rnm(1)~=m)||(Rnm(2)~=m);
WM{3}=(Nnm(1)~=n)||(Nnm(2)~=m);
if (any(X(:)))
    WM{4}=(Xnm(1)~=n)||(Xnm(2)~=n);
else
    WM{4}=0;
end
if isscalar(CS)~=1
    if sum(sum(abs(CS) >1))
        error(errormsg{15});
    end    
    WM{5}=(CSnm(1)~=m)||(CSnm(2)~=l);
else
    WM{5}=0;
end

emt=[9,10,11,12,14;
     16,17,18,19,20;
     21,22,23,24,25;
     26,27,28,29,30;
     31,32,33,34,35;
     36,37,38,39,40;
     41,42,43,44,45;
     46,47,48,49,50;
     51,52,53,54,55;
     56,57,58,59,60];
 
for j=1:5
    if WM{j}
        if strcmp(ct,'p')
            error(errormsg{emt(1,j)});
        elseif strcmp(ct,'pi')
            error(errormsg{emt(2,j)});
        elseif strcmp(ct,'pid')
            error(errormsg{emt(3,j)});
        elseif strcmp(ct,'pd')
            error(errormsg{emt(4,j)});
        elseif strcmp(ct,'d')
            error(errormsg{emt(5,j)});
        elseif strcmp(ct,'dof')
            error(errormsg{emt(6,j)});
        elseif strcmp(ct,'dofi')
            error(errormsg{emt(7,j)});
        elseif strcmp(ct,'dofid')
            error(errormsg{emt(8,j)});
        elseif strcmp(ct,'dofd')
            error(errormsg{emt(9,j)});
        else
            error(errormsg{emt(10,j)});
        end
    end
end
% Testing the positive definiteness of the X matrix
if (any(X(:)))
    if min(eig(X))<=0
        error(errormsg{13});
    end
end

% Testing other dependencies
for j=1:p
    if any(Ds{j}(:))~=0
        error(errormsg{61});
    end
end
if min(eig(Q-N/R*N'))<-1e-10
    error(errormsg{62});
end
if min(eig(R))<=0
    error(errormsg{63});
end
if eps_s<=0
    error(errormsg{64});
end
if isempty(which('yalmip.m'))
    error(errormsg{65});
end

%% Calculating the initial Lyap. matrix
if any(X(:))==0
    An=zeros(n,n);
    Bn=zeros(n,m);
    for j=1:p
        An=An+A{j}*1/p;
        Bn=Bn+B{j}*1/p;
    end
    [~,X,~]=lqr(An,Bn,Q,R,N);
end

%% CONTROLLER DESIGN
tic;
if verbose>0
    fprintf('Output-feedback LQR design (oflqr)\n');
    fprintf('Iterative controller design has started:\n');
    if verbose>1
        fprintf('  Iter. | Epsilon    | Iter. time | srv  \n');
    end
end

iterr=1;
lminok=true;
sObj=trace(X);
nlop=true;
fcount=0;
while ((iterr<=itern)&&(lminok))&&(nlop)
    clear P F ;

    F=sdpvar(m,l,'full');
    
    [Fn,Fm]=size(F);
    if isscalar(CS)~=1
        for j=1:Fn
            for k=1:Fm
                if CS(j,k)~=1
                    F(j,k)=0;
                end
            end
        end
    end

    P=sdpvar(n,n);
    
    Ms=cell(1,p);
    for j=1:p
        G=F*C{j}-R\(B{j}'*P+N');
        M=[A{j}'*P+P*A{j}+Q-(X*B{j}+N)/R*(B{j}'*P+N')-(P*B{j}+N)/R*(B{j}'*X+N')+(X*B{j}+N)/R*(B{j}'*X+N'),G';
              G,-inv(R)];
        Ms{j}=M<=0;
    end
    
    Constraints=[Ms{1:end},P>=epsP*eye(n)];

    if any(x0)
        Objective=x0'*P*x0;
    else
        Objective=trace(P);
    end
    opts=sdpsettings(solverOpt{1:end},'verbose',0);
    sol=optimize(Constraints,Objective,opts);        % Call the solver
    
    err=abs(sObj-double(Objective));
    if (sol.problem==0)&&(err<=eps_s)
        lminok=false;
    elseif (sol.problem~=0)&&(err<=eps_s)
        fcount=fcount+1;
        if fcount>mlop
            nlop=false;        
        end
    end
        
    
    X=double(P);
    if verbose>1
        fprintf(' %6d |%11.4e |%11.4e |%2d \n',iterr,err,sol.yalmiptime,sol.problem);
    end
    
    sObj=double(Objective);
    iterr=iterr+1;
end

% Calculating the outputs
iterr=iterr-1;
rtime=toc;
[primf,dualf]=check(Constraints);
Er=zeros(n,1,p);
Fv=double(F);
for j=1:p
    Ac=A{j}-B{j}*Fv*C{j};
    Er(:,:,j)=eig(Ac);
end
%% DISPLAYING THE RESULTS

if (p>1)&&(ms==1)&&(ls==1)
    textrs='Robust ';
elseif (p>1)&&(isscalar(CS))&&((ms~=1)||(ls~=1))
    textrs='Centralized robust ';
elseif (p>1)&&(isscalar(CS)~=1)
    textrs='Structured robust ';
elseif (p<2)&&(isscalar(CS)~=1)
    textrs='Strucutred ';
elseif (p<2)&&(isscalar(CS))&&((ms~=1)||(ls~=1))
    textrs='Centralized ';
else
    textrs='';
end
if strcmp(ct,'p')
    contstrut=sprintf(['\n%sP controller structure:\n',...
        '       u = - Kp y\n\n',...
        '       F(%dx%d) = [Kp(%dx%d)]\n'],...
        textrs,m,l,m,l);
elseif strcmp(ct,'pi')
    contstrut=sprintf(['\n%sPI controller structure:\n',...
        '       u = - Kp y - Ki Integral yi\n\n',...
        '       F(%dx%d) = [Kp(%dx%d)  Ki(%dx%d)]\n'],...
        textrs,m,l,ms,ls,ms,li);        
elseif strcmp(ct,'pid')
    contstrut=sprintf(['\n%sPID controller structure:\n'...
        '       u = - Kp y - Ki Integral yi - Kd dydf/dt\n\n',...
        '       where dydf/dt is the vector of filtered derivatives, using\n',...
        '       derivative filter Gf(s)=Nf s/(s+Nf). (Nf=%.3E)\n\n',...
        '       F(%dx%d) = [Kp(%dx%d)  Ki(%dx%d)  Kd(%dx%d)]\n'],...
        textrs,Nf,m,l,ms,ls,ms,li,ms,ld);
elseif strcmp(ct,'pd')
    contstrut=sprintf(['\n%sPD controller structure:\n',...
        '       u = - Kp y - Kd dydf/dt\n\n',...
        '       where dydf/dt is the vector of filtered derivatives, using\n',...
        '       derivative filter Gf(s)=Nf s/(s+Nf). (Nf=%.3E)\n\n',...
        '       F(%dx%d) = [Kp(%dx%d)  Kd(%dx%d)]\n'],...
        textrs,Nf,m,l,ms,ls,ms,ld);
elseif strcmp(ct,'d')
    contstrut=sprintf(['\n%sD controller structure:\n',...
        '       u = - Kd dydf/dt\n\n',...
        '       where dydf/dt is the vector of filtered derivatives, using\n',...
        '       derivative filter Gf(s)=Nf s/(s+Nf). (Nf=%.3E)\n\n',...
        '       F(%dx%d) = [Kd(%dx%d)]\n'],...
        textrs,Nf,m,l,ms,ld);
elseif strcmp(ct,'dof')
    contstrut=sprintf(['\n%sDOF controller structure:\n',...
        '     dxc = - Ac xc - Bc y\n',...
        '       u = - Cc xc - Dc y\n\n',...
        '      F(%dx%d) =  [ Dc(%dx%d)  Cc(%dx%d) ]\n',...
        '                [ Bc(%dx%d)  Ac(%dx%d) ]\n'],...
        textrs,nc+ms,nc+ls,ms,ls,ms,nc,nc,ls,nc,nc);
elseif strcmp(ct,'dofi')
    contstrut=sprintf(['\n%sDOFI controller structure:\n',...
        '     dxc = - Ac xc - Bc1 y - Bc2 Integral yi\n',...
        '       u = - Cc xc - Dc1 y - Dc2 Integral yi\n\n',...
        '      F(%dx%d) =  [ Dc1(%dx%d)  Cc(%dx%d)  Dc2(%dx%d) ]\n',...
        '                [ Bc1(%dx%d)  Ac(%dx%d)  Bc2(%dx%d) ]\n'],...
        textrs,m,n,ms,ls,ms,nc,ms,li,nc,ls,nc,nc,nc,li);
elseif strcmp(ct,'dofid')
    contstrut=sprintf(['\n%sDOFID controller structure:\n',...
        '     dxc = - Ac xc - Bc1 y - Bc2 Integral yi - Bc3 dydf/dt\n',...
        '       u = - Cc xc - Dc1 y - Dc2 Integral yi - Dc3 dydf/dt\n\n',...
        '       where dydf/dt is the vector of filtered derivatives, using\n',...
        '       derivative filter Gf(s)=Nf s/(s+Nf). (Nf=%.3E)\n\n',...
        '      F(%dx%d) =  [ Dc1(%dx%d)  Cc(%dx%d)  Dc2(%dx%d)  Dc3(%dx%d) ]\n',...
        '                [ Bc1(%dx%d)  Ac(%dx%d)  Bc2(%dx%d)  Bc3(%dx%d) ]\n'],...
        textrs,Nf,m,l,ms,ls,ms,nc,ms,li,ms,ld,nc,ls,nc,nc,nc,li,nc,ld);
elseif strcmp(ct,'dofd')
    contstrut=sprintf(['\n%sDOFD controller structure:\n',...
        '     dxc = - Ac xc - Bc1 y - Bc3 dydf/dt\n',...
        '       u = - Cc xc - Dc1 y - Dc3 dydf/dt\n\n',...
        '       where dydf/dt is the vector of filtered derivatives, using\n',...
        '       derivative filter Gf(s)=Nf s/(s+Nf). (Nf=%.3E)\n\n',...
        '      F(%dx%d) =  [ Dc1(%dx%d)  Cc(%dx%d)  Dc3(%dx%d) ]\n',...
        '                [ Bc1(%dx%d)  Ac(%dx%d)  Bc3(%dx%d) ]\n'],...
        textrs,Nf,m,l,ms,ls,ms,nc,ms,ld,nc,ls,nc,nc,nc,ld);
end    

if (iterr>=itern)||(sol.problem==0)
    if (iterr>=itern)
        rv=1;
        if (sol.problem==0)
            restext=sprintf(['WARNING!\n',...
                'Optimization was stopped by the iteration counter!\n',...
                'The result may be wrong!\n']);                
        else
            restext=sprintf(['WARNING! Controller design failed.\n',...
                'Optimization was stopped by the iteration counter!\n']);  
        end
    end
    if ~lminok
        rv=0;
        restext=sprintf('Optimization was SUCCESSFUL!\n');        
    end    
elseif ((iterr<=itern)&&(lminok))&&(nlop)
    rv=2;
    disp('WARNING! Controller design failed.');
    disp('Something went wrong.');
elseif ~nlop
    rv=4;
    restext=sprintf('WARNING! Controller design failed.\nLack of progress.\n');   
else
    rv=-1;
    restext=sprintf('WARNING! Controller design failed.\nUnknown problem.\n');
end

if verbose==1
    fprintf('  Iter. | Epsilon    | Time       | rv  \n');
    fprintf(' %6d |%11.4e |%11.4e | %d \n',iterr,err,rtime,rv); 
    fprintf(restext);
end
if verbose>1
    fprintf('Finished. Overal time: %.4e sec.\n',rtime);
    fprintf(restext);
    if ((rv==0)||(rv==1))&&(verbose>2)
        fprintf(contstrut);
    end
end

Fr=double(F);
Pr=double(P);


%% Creating INFO output
% INFO INPUTS
designinfo.inputs.iter.info=sprintf('Maximal number of iterations.');
designinfo.inputs.iter.value=itern;
designinfo.inputs.eps.info=sprintf('Epsilon for stopping criteria.');
designinfo.inputs.eps.value=eps_s;
designinfo.inputs.X.info=sprintf('Initial Lyapunov matrix.');
designinfo.inputs.X.value=X;
designinfo.inputs.nc.info=sprintf('Number of states for the dyamic controller.');
designinfo.inputs.nc.value=nc;
designinfo.inputs.CS.info='Controller structure matrix.';
designinfo.inputs.CS.value=CS;
designinfo.inputs.settings.info='Options structure for Yalmip.';
designinfo.inputs.settings.value=solverOpt;
designinfo.inputs.epsP.info='Epsilon for positive definiteness test of P.';
designinfo.inputs.epsP.value=epsP;
designinfo.inputs.Nf.info='Filter constant for the derivative filter.';
designinfo.inputs.Nf.value=Nf;
designinfo.inputs.Ci.info='Output matrix for Integral part.';
designinfo.inputs.Ci.value=Ci;
designinfo.inputs.Cd.info='Output matrix for Derivative part.';
designinfo.inputs.Cd.value=Cd;
designinfo.inputs.verbose.info='Display information control value.';
designinfo.inputs.verbose.value=verbose;
designinfo.inputs.ct.info='Controller type.';
designinfo.inputs.ct.value=ct;
designinfo.inputs.sysprop.n.info='Number of states.';
designinfo.inputs.sysprop.n.value=ns;
designinfo.inputs.sysprop.m.info='Number of inputs.';
designinfo.inputs.sysprop.m.value=ms;
designinfo.inputs.sysprop.l.info='Number of outputs.';
designinfo.inputs.sysprop.l.value=ls;
designinfo.inputs.sysprop.li.info='Number of outputs for the integral part.';
designinfo.inputs.sysprop.li.value=li;
designinfo.inputs.sysprop.ld.info='Number of outputs for the derivative part.';
designinfo.inputs.sysprop.ld.value=ld;
designinfo.inputs.mlop.info='Maximal number if iterations before stopping by lack of progress warning.';
designinfo.inputs.mlop.value=mlop;
% INFO OUTPUTS
designinfo.outputs.rv.info='Return value.';
designinfo.outputs.rv.value=rv;
designinfo.outputs.iter.info='Number of iterations.';
designinfo.outputs.iter.value=iterr;
designinfo.outputs.rtime.info='Run-time.';
designinfo.outputs.rtime.value=rtime;
designinfo.outputs.objer.info='Final objective error.';
designinfo.outputs.objer.value=err;
designinfo.outputs.primf.info='Primal feasibility values.';
designinfo.outputs.primf.value=primf;
designinfo.outputs.dualf.info='Dual feasibility values.';
designinfo.outputs.dualf.value=dualf;
designinfo.outputs.designinfo.info='Information about the controller design.';
designinfo.outputs.designinfo.value=restext;
designinfo.outputs.ctrstructure.info='Controller structure.';
designinfo.outputs.ctrstructure.value=contstrut;

end
